/*
   $Id: rpmmen.c,v 1.26 2001/11/04 13:36:36 pawelk Exp $
 */
/* PLD instalator - rpm manager module (C) Pawe Koodziej 
   please send any comments, suggestions and bugs reports to me
   (pawelk@pld.org.pl)
   GNU license
 */



#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <rpm/rpmlib.h>
#include <rpm/rpmmacro.h>
#include "utils.h"
#include <trurl/trurl.h>
#include "rpmmenlib.h"
#include "cfile.h"

int rpmpkgs_preparetoinstall(TPkgSet * pkgs);

/* select_from_file : read names of packages to install from file
ARG  : name - file name
 */
void rpmpkgs_select_from_file(TPkgSet * pkgs, char *name)
{
	int npkgs;
	char buf[PATH_MAX], *p;
	FILE *f;

	f = fopen(name,"r");
	if (f == NULL) {
		log_message(_("Can't open file %s"), name);
		return;
	}

	n_array_sort(pkgs->packages);
	npkgs = pkgs_array_num(pkgs);

	while (fgets(buf, sizeof(buf), f)) {
		int n, base_head = 0;
		TPkgInfo *pkg, tmppkg;

		n = strlen(buf);

		buf[n-1] = '\0';
		if (buf[n-2] == '\r') 
			buf[n-2] = '\0';

		p = buf;
		while(isspace(*p))
			p++;

		if ((*p && *p == '#') || *p == '\0')
			continue;

		if (*p == '*') {
			base_head = 1;
			p++;
		}

		tmppkg.name = p;
		if ((pkg = n_array_bsearch(pkgs->packages, &tmppkg))) {
			log_debug(_("pkg %s"), p);
			pkg->selected = RPMMEN_FROM_FILE;
			/*	
			pkg->flags |= TPKGINF_BASESYS; 
			*/
			if (base_head)
				pkg->flags |= TPKGINF_BASESYS_HEAD | 
						TPKGINF_BASESYS;
			
		}
	}

	fclose(f);
}

/* rpmpkgs_init: Init rpmmen lib
 */
void rpmpkgs_init(TPkgSet *pkgs, char *src, char *dest_, struct rpmmen_cb *cb)
{
/*	rpmSetVerbosity(0);      */
	memset(pkgs, 0, sizeof(TPkgSet));
	/*      pkgs->packages = pkgs_array_new(); */
	/*Q     pkgs->numPackages = 0; */
	pkgs->src = strdup(src);
	if(!dest_)
		dest_="";
	pkgs->rootdir = strdup(dest_);
	pkgs->dbpath = strdup(dest_);
	pkgs->db=NULL;
	pkgs->cb = cb;
}

void rpmpkgs_free(TPkgSet * pkgs)
{
	if (pkgs->packages)
		n_array_free(pkgs->packages);
	if (pkgs->src)
		free(pkgs->src);
	if (pkgs->rootdir)
		free(pkgs->rootdir);
	if (pkgs->dbpath)
		free(pkgs->dbpath);

	/*Q     pkgs->numPackages = 0; */
}


/* rpmmen_mkdir - creat directory, log erros
ARG : d -directory name
RET : 1 - succes (directory created or still exist)
0 - fail (errno is set appropriately)
STAT : internal function
 */
int rpmmen_mkdir(char *d)
{
	if (mkdir(d, 0755))
		if (errno != EEXIST) {
		   log_message(_("Fail to creat directory %s because :%s"), \
					d, strerror(errno));
			return 0;
		}
	return 1;
}

/* rpmmen_opendb - create directory for base, and open (creat if not exist) rpm
   database files

RET: 0 - OK
1 - fail - can't create directory for base
2 - fail - can't open (create) base
 */
int rpmpkgs_opendb(TPkgSet * pk)
{
	int st;
	char tmp[PATH_MAX];

	strcpy(tmp, pk->rootdir);
	strcat(tmp, RPMMEN_DBPATH);
	if (!mkdirhier(tmp)) {
		log_message(_("rpmmen: can't create %s for rpm base"), tmp);
		return 1;
	}
	log_debug(_("mkdir'ed path: %s"), tmp);
	log_debug(_("rootdir: %s"), pk->rootdir);

	rpmReadConfigFiles(NULL, NULL);
/*	addMacro(NULL, "_dbpath", NULL, RPMMEN_DBPATH, RMIL_DEFAULT);
*/
	st = rpmdbInit(pk->dbpath, 0644);	/* it doesn't delete existing base */
	if (st) {
		log_message(_("%s: can't create database %s: %s"), __FUNCTION__,
				pk->dbpath, strerror(errno));
		return 2;	/* FAIL */
	}
	if (rpmdbOpen(pk->dbpath, &(pk->db), O_RDWR, 0644)){
		pk->db=NULL;	
		return 2;
	}

	return 0;
}

int rpmpkgs_opendb_ro(TPkgSet * pk)
{
	char tmp[PATH_MAX];

	rpmReadConfigFiles(NULL, NULL);
/*	addMacro(NULL, "_dbpath", NULL, RPMMEN_DBPATH, RMIL_DEFAULT);
*/
	if (rpmdbOpen(pk->dbpath, &(pk->db), O_RDONLY, 0644)){
		pk->db=NULL;	
		return 2;
	}

	return 0;
}

void EVRget(Header h, int release, char *buf)
{
        int *epoch;
        char *version,*revision;
        int counte,countr;
        strcpy(buf,"");
        headerGetEntry(h,RPMTAG_EPOCH,NULL,(void **)&epoch,&counte);
        headerGetEntry(h,RPMTAG_VERSION,NULL,(void **)&version,NULL);
        headerGetEntry(h,RPMTAG_RELEASE,NULL,(void **)&revision,&countr);
        if(counte){
                char ebuf[10];
                sprintf(ebuf,"%d",*epoch);
                strcat(buf,ebuf);
                strcat(buf,":");
        }
        strcat(buf,version);

        if(release){
                strcat(buf,"-");
                strcat(buf,revision);
        }
}


void EVRgetSplited(Header h, int *e,char *v, char *r)
{
	int *epoch;
	char *version,*revision;
	int counte;
	
	headerGetEntry(h,RPMTAG_EPOCH,NULL,(void **)&epoch,&counte);
	if(counte)
		*e=*epoch;
	else
		*e=-1;
	
	headerGetEntry(h,RPMTAG_VERSION,NULL,(void **)&version,NULL);
	strcpy(v,version);
	headerGetEntry(h,RPMTAG_RELEASE,NULL,(void **)&revision,NULL);
	strcpy(r,revision);
}



/** if (release) compare full EVR, if (!release) comapre only EV. */
int EVRcmp(Header a,Header b,int release)
{
	char v1[200],v2[200],r1[200],r2[200];
	int e1, e2;
	int rc;

	EVRgetSplited(a,&e1, v1, r1);
	EVRgetSplited(b,&e2, v2, r2);
	
	if(e1 != e2)
		return e1-e2;
	rc = rpmvercmp(v1,v2);
	if(rc != 0 || release==0)
		return rc;
	return rpmvercmp(r1,r2);

}

/* rpmmen_scandir - scan directory for packages
RET :  0  - OK
1  - fail - can't get files list
 */
int rpmpkgs_scandir(TPkgSet * pkgs)
{
	TPkgSet *tmp;
	int i;

	tmp = getPackagesList(pkgs->local_tocfile);

	if (tmp == NULL) {
		return 1;
	}
	pkgs->packages = tmp->packages;		/* rzebienie palcem */
	/*      pkgs->numPackages = tmp->numPackages; *//* w miekim materiale - PK */

	
	for(i=0;i<pkgs_array_num(pkgs);i++)
		(TPkgInfo *)pkgs_array_nth(pkgs,i)->installedEVR=NULL;
		
	/* check if package can be used for upgrade */
	 rpmpkgs_update_info(pkgs);
	return 0;
}

/* rpmmen_  - close rpm base */
void rpmpkgs_close(TPkgSet * pkgs)
{
	if (pkgs->db)
		rpmdbClose(pkgs->db);
	pkgs->db=NULL;
}

void addNeededBy(TPkgInfo *down,TPkgInfo *up)
{
	if( up == NULL || down == NULL){
		log_debug("%s: null pointer", __FUNCTION__);
		return;
	}
	if(down->neededBy == NULL)
		down->neededBy = n_list_new(TN_LIST_UNIQ, NULL, NULL);
	n_list_push(down->neededBy, up);
}

void printNeededBy(TPkgSet *pkgs)
{
	TPkgInfo *pk, *pkB;
	
	int i,j;
	for(i=0;i<pkgs_array_num(pkgs); i++){
		pk = pkgs_array_nth(pkgs, i);
		if(pk->neededBy == NULL)
			continue;
		log_message("PKG %s is required by:",pk->name);
		for(j=0; j < n_list_size(pk->neededBy); j++){
			pkB = (TPkgInfo *)n_list_nth(pk->neededBy,j);
			log_message("== %s",pkB->name);
		}
	}
}
			
			
			

/* preparetoinstall - find dependencies and select necessary packages to 
   install
RET  : number of selected (from dependencies) packages
STAT : internal function
 */

int rpmpkgs_preparetoinstall(TPkgSet * pkgs)
{
	rpmTransactionSet rpmdep;
	int i, num_conflicts, numConflicts;
	struct rpmDependencyConflict *conflicts;
	TPkgInfo *pk;
	int manualy=0;
	int update;
	/* XXX: dirty, but rpmlib seems to require this */
	rpmpkgs_close(pkgs);
	rpmpkgs_opendb(pkgs);
	
	rpmdep = rpmtransCreateSet(pkgs->db, /*pkgs->rootdir */ NULL);

	if(pkgs->cb->deps_prog)
		pkgs->cb->deps_prog(0, 0, pkgs_array_num(pkgs), _("Checking deps..."));
		
	for (i = 0; i < pkgs_array_num(pkgs); i++) {
		pk = pkgs_array_nth(pkgs, i);
		if (pk->flags & TPKGINF_HASHDR) {
			if (pk->selected || pk->flags & TPKGINF_BASESYS){
				update=((TPKGINF_UPGRADE|TPKGINF_EQUAL|
							TPKGINF_OLDER))?1:0;
				rpmtransAddPackage(rpmdep, pk->h, NULL, pk,
						update,NULL);
			}
			else {
				rpmtransAvailablePackage(rpmdep, pk->h, pk);
			}
		}
		if (i % 30 == 0) {
			log_debug("-> %d", i);
			if(pkgs->cb->deps_prog)
				pkgs->cb->deps_prog(1, i, 0, 0);
		}

	}
#if 0
	do{
#endif
		numConflicts=0;
		if(pkgs->cb->deps_prog)
			pkgs->cb->deps_prog(0, 0, 0, _("Checking conflicts..."));
		
		rpmdepCheck(rpmdep, &conflicts, &numConflicts);
		if(pkgs->cb->deps_prog)
			pkgs->cb->deps_prog(0, 0, numConflicts, _("Checking conflicts..."));
		    
		log_message(_("Next step"));
		num_conflicts = 0;	/* Konfilkty ktre dooyy pakiety */
		for (i = 0; i < numConflicts; i++) {
			if(pkgs->cb->deps_prog)
				pkgs->cb->deps_prog(1, i, 0, 0);
			if(conflicts[i].suggestedPackages)
				pk = (TPkgInfo *) conflicts[i].suggestedPackages[0];
			else
				pk = NULL;
			if (conflicts[i].sense == RPMDEP_SENSE_REQUIRES) {
				if(pkgs->cb->selectOne && conflicts[i].suggestedPackages &&
					conflicts[i].suggestedPackages[0] &&
					conflicts[i].suggestedPackages[1]){
					int s, selectOne=1;
					TPkgInfo *tmpPkg;
					for(s=0;conflicts[i].suggestedPackages[s];s++){
						tmpPkg=(TPkgInfo *) conflicts[i].suggestedPackages[s];
						if(tmpPkg->selected){					
							selectOne=0;
							pk = tmpPkg;
							log_message("multi suggestion, but %s is selected"
									"already",pk->name);
						}
					}
					if(selectOne)					
						pk=pkgs->cb->selectOne(conflicts + i);
				}
				if (pk != NULL) {
/*					if(!(pk->selected)){
						update=( 1 &(TPKGINF_UPGRADE|
									TPKGINF_EQUAL|
									TPKGINF_OLDER))?1:0;
						log_message("OK - DODAJE DO SETa'a %p",pk);
						rpmtransAddPackage(rpmdep,pk->h,
								NULL,pk,update,NULL); 
					}
*/
					pk->selected = RPMMEN_FROM_DEPS;
					addNeededBy(pk,conflicts[i].byKey);
					num_conflicts++;
					log_message(_("%d added %s as a consequence "
							"of dependecies"), num_conflicts, pk->name);
				}
				else{
					if(pkgs->cb->conflict)
						pkgs->cb->conflict(
								RPMMEN_CB_CONFLICT_UNSATISFIED,
								conflicts[i].byName,
								conflicts[i].byVersion,
								conflicts[i].byRelease,
								conflicts[i].needsName,
								conflicts[i].needsVersion,
								conflicts[i].needsFlags);

					manualy++;
					log_message(_("unsatisfied dependence"));
				}
			} else {
				log_message(_("There is dependencies confilc which "
						"can't be resolved"));
				if(pkgs->cb->conflict)
					pkgs->cb->conflict(RPMMEN_CB_CONFLICT_ADD,
							conflicts[i].byName,
							conflicts[i].byVersion,
							conflicts[i].byRelease,
							conflicts[i].needsName,
							conflicts[i].needsVersion,
							conflicts[i].needsFlags);
				manualy++;
			}
		}
		if (conflicts)
			rpmdepFreeConflicts(conflicts, numConflicts);
#if 0
	}while(num_conflicts /*&& !manualy*/);
#endif
			
	rpmtransFree(rpmdep);
	if (pkgs->cb->deps_prog)
		pkgs->cb->deps_prog(2, 0, 0, 0);

	return ((!num_conflicts)?(-manualy):num_conflicts);

}

void doGetAllUpTo(TPkgInfo *pk, tn_list *list){
	int i;
	n_list_push(list,pk);
	if(pk->neededBy == NULL)
		return;
	for(i=0; i< n_list_size(pk->neededBy) ; i++)
		doGetAllUpTo((TPkgInfo *)n_list_nth(pk->neededBy, i), list);
	return;
}

tn_list* getAllUpTo(TPkgInfo *pk){
	tn_list *ret = n_list_new(TN_LIST_UNIQ, NULL, NULL);
	doGetAllUpTo(pk,ret);
	return ret;
}

/** return list of lists 
 *  first element of each list is package that causes problem becouse it's
 *  on black list
 **/
tn_list* rpmpkgs_checkBlack(TPkgSet *pkgs)
{
	int i;
	TPkgInfo *pk;
	tn_list * ret = NULL;

	for(i=0;i<pkgs_array_num(pkgs); i++){
		pk = pkgs_array_nth(pkgs, i);
		if(! (pk->flags & TPKGINF_BLACK))
			continue;
		if(! (pk->selected & RPMMEN_FROM_DEPS))
			continue;
		if(ret == NULL)
			ret = n_list_new(TN_LIST_UNIQ, NULL, NULL);
		n_list_push(ret, getAllUpTo(pk));
	}
	return ret;
}			

void rpmmen_freeBlack(tn_list *list)
{
	int i;
	
	if(list == NULL)
		return;
	for(i=0;i<n_list_size(list);i++)
		n_list_free((tn_list *)n_list_nth(list,i));
	n_list_free(list);
}
	
	

/* rpmmmen_addPkgsDep  -- add pagages from dependencies (selected = 64)
 * RET: number of conflicts
 */
int rpmpkgs_addPkgsDep(TPkgSet * pkgs)
{
	int status;

	if(pkgs->cb->conflict)
		pkgs->cb->conflict(RPMMEN_CB_CONFLICT_NEW,NULL,NULL,NULL,NULL,
				NULL,0);
	while ((status=rpmpkgs_preparetoinstall(pkgs))>0)
		/* nothing */ ;
	if(pkgs->cb->deps_prog)
		pkgs->cb->deps_prog(2, 0, 0, 0);

	if(pkgs->cb->conflict)		
		pkgs->cb->conflict(RPMMEN_CB_CONFLICT_SHOW,NULL,NULL,NULL,NULL,
				NULL,0);
	printNeededBy(pkgs);
	return(status<0?-status:0);
}

void rpmpkgs_removePkgsDep(TPkgSet * pkgs)
{
	int i;
	TPkgInfo *pk;
	for (i = 0; i < pkgs_array_num(pkgs); i++) {
		pk = pkgs_array_nth(pkgs, i);
		if (pk->selected == RPMMEN_FROM_DEPS)
			pk->selected = 0;
		if(pk->neededBy){
			n_list_free(pk->neededBy);
			pk->neededBy = NULL;
		}
	}

}


void *rpmmen_callback( const void * h_v ,
		const rpmCallbackType what,
		const unsigned long amount,
		const unsigned long total,
		const void *key,
		void *data)
{
	TPkgSet      *pkgs;
	char         *fname = NULL;
	static FD_t  fdt;
	int          fd;
	Header h=(Header) h_v;
	
	pkgs = (TPkgSet *) data;

	if (key)
		fname = ((TPkgInfo *) key)->fname;

	switch (what) {

		case RPMCALLBACK_INST_OPEN_FILE:
			log_debug(_("%d INST_OPEN_FILE %ld %ld"), (int) what,
					amount, total);
			if (pkgs->cb && pkgs->cb->pkg_open)
				pkgs->cb->pkg_open(key);

			fd = psOpenPkg(&(PsSet.ps[((TPkgInfo*)key)->ps_i] ) ,fname);
			if (fd < 0) {
				log_message(_("open %s failed: %s, skipped"), fname,
						"BO COSTAM");
				fdt = NULL;

			} else {
				fdt=fdDup(fd); 
				close(fd);	/* one memory and
						 * fd leak killed --mal */
			/*	fdSetFdNo(fdt, fd); */
				fprintf(stderr, _("\ninstall %s\n"), fname);
			}

			return fdt;

		case RPMCALLBACK_INST_CLOSE_FILE:
			log_debug(_("%d INST_CLOSE_FILE %ld %ld"), (int) what,
					amount, total);

			if (fdt) {
				psClosePkg(fdFileno(fdt));
				fdFree(fdt, "");
			}

			if (pkgs->cb && pkgs->cb->pkg_close)
				pkgs->cb->pkg_close(key);

			break;


		case RPMCALLBACK_INST_START:
			log_message(_("installing %s"), fname);
			break;

		case RPMCALLBACK_INST_PROGRESS:
			if (pkgs->cb && pkgs->cb->pkg_inst_progress)
				pkgs->cb->pkg_inst_progress(amount, total);
			break;

		case RPMCALLBACK_TRANS_START:
			break;

		case RPMCALLBACK_UNINST_START:
			break;

		case RPMCALLBACK_TRANS_PROGRESS:
			break;
		case RPMCALLBACK_UNINST_PROGRESS:
			break;

		case RPMCALLBACK_TRANS_STOP:
			break;

		case RPMCALLBACK_UNINST_STOP:
			break;

		default:
			log_message(_("callback unknown action in %s"), __FUNCTION__);
	}

	return NULL;
}



/* rpmmen_install - install selected packages 
ARGS : flags - flags for rpmInstallPackage (called from installrpm)
notify - pointer to function called from rpmInstallPAckage during 
package installation
startUI - pointer to function called before each package installation
RET  : 0 - OK (it's only one possible value)
 */
int rpmpkgs_install(TPkgSet * pkgs, int flags	/*, rpmNotifyFunction notify,
						  TStartPackage startUI */ )
{
	rpmTransactionSet rpmorder;
	TPkgInfo /* **order, */  * pk;
	int i, npkgs;
	rpmProblemSet pro;
	int update;


	rpmorder = rpmtransCreateSet(pkgs->db, pkgs->rootdir);

	npkgs = pkgs_array_num(pkgs);
	for (i = 0; i < npkgs; i++) {
		pk = pkgs_array_nth(pkgs, i);

		if (pk->selected && pk->flags & TPKGINF_BASESYS_HEAD) {
			log_debug(_("basesys_atfirst %s"), pk->name);	
			update=(pk->flags&(TPKGINF_UPGRADE|TPKGINF_EQUAL|
						TPKGINF_OLDER))?1:0;
			rpmtransAddPackage(rpmorder, pk->h, NULL, pk, 
					update, NULL);
			pk->selected = RPMMEN_FROM_FILE;
		}
	}


	for (i = 0; i < npkgs; i++) {
		pk = pkgs_array_nth(pkgs, i);
		if (pk->selected && pk->flags & TPKGINF_BASESYS &&
				!(pk->flags & TPKGINF_BASESYS_HEAD)) {
			log_debug(_("basesys %s"), pk->name);
			update=(pk->flags&(TPKGINF_UPGRADE|TPKGINF_EQUAL|
						TPKGINF_OLDER))?1:0;
			rpmtransAddPackage(rpmorder, pk->h, NULL, pk,update,NULL);
			pk->selected = RPMMEN_FROM_FILE;
		}
	}


	for (i = 0; i < npkgs; i++) {
		pk = pkgs_array_nth(pkgs, i);
		if (pk->flags & TPKGINF_BASESYS)
			continue;

		if (pk->selected) {
			log_debug(_("selected %s"), pk->name);
			update=(pk->flags&(TPKGINF_UPGRADE|TPKGINF_EQUAL|
						TPKGINF_OLDER))?1:0;
			rpmtransAddPackage(rpmorder,pk->h,NULL,pk,update,NULL);
			if (!(pk->flags & TPKGINF_HASHDR)) 
				log_message(_("selected %s hasn't header"),
						pk->name);
		}
	}


	i = rpmdepOrder(rpmorder);
	log_debug(_("ordering result: %d"), i);

/* fetch all packages from net before installing them !*/
	for (i = 0; i < npkgs; i++) {
		pk = pkgs_array_nth(pkgs, i);
		if(pk->selected)
			psFetchPkg(PsSet.ps+pk->ps_i,pk->fname);
	}

	rpmRunTransactions(rpmorder, rpmmen_callback, pkgs, NULL, &pro, flags, ~0);
	rpmtransFree(rpmorder);

	return 0;
}

/* printHeadre - print some info from package header
   STAT - not used
 */
void printHeader(Header h)
{
	int i, count;
	char *name;
	headerGetEntry(h, RPMTAG_NAME, &i, (void **) &name, &count);
	printf("#%s#\n", name);
	/* nie powinno byc free(name) ???? (w rpm nie ma) */
}



void *rpmpkg_header_entry_get(TPkgSet *pkgs, int num, int tag)
{
	void *tmp;
	TPkgInfo *pkg;

	pkg = pkgs_array_nth(pkgs, num);
	if (pkg == NULL)
		return NULL;

	headerGetEntry(pkg->h, tag, NULL, &tmp, NULL);
	return tmp;

}

void rpmpkgs_header_entry_free(void *ptr)
{
}


unsigned long int rpmpkgs_selected_size(const TPkgSet *pkgs,
		int *nselected_pkgs)
{
	void *tmp;
	TPkgInfo *pkg;
	unsigned long size = 0;
	int i, n, npkgs = 0;

	n = pkgs_array_num(pkgs);
	for (i = 0; i < n; i++) {
		int ps = 0;
		pkg = pkgs_array_nth(pkgs, i);

		if (pkg->selected) {
			headerGetEntry(pkg->h, RPMTAG_SIZE, NULL, &tmp, NULL);
			ps = *(int*)tmp;
			log_debug(_("ADD %s %d, %d, all %ld"), pkg->name, npkgs, ps,  size);
			if (ps > 0) 
				size += ps;
			npkgs++;
		}
	}
	*nselected_pkgs = npkgs;
	return size;
}

void rpmpkgs_selectpackages(TPkgSet *pkgs,char **names,int count)
{
	TPkgInfo tmppk,*pk;
	int i;
	/* XXX assume that packages list is sorted */
	for(i=0;i<count;i++){
		tmppk.name=names[i];
		pk=n_array_bsearch(pkgs->packages,&tmppk);
		if(pk)
			pk->selected=RPMMEN_FROM_FILE;
	}
}

void rpmpkgs_update_info(TPkgSet *pkgs)
{
	TPkgInfo *pkg;
	int i;
	Header h1=NULL;
	int cmp,cmpr;
	int maxi;
	char tmpbuf[100];
	rpmdbMatchIterator mi;


	maxi=pkgs_array_num(pkgs);
	for(i=0;i<maxi;i++){
		h1=NULL;
		pkg=pkgs_array_nth(pkgs,i);
		/* FIXME what if there are more then one package with that 
		 * name ?*/
		mi=rpmdbInitIterator(pkgs->db,RPMTAG_NAME,pkg->name,0);
		if(mi){
			h1=rpmdbNextIterator(mi);
		}
		pkg->flags &= ~(TPKGINF_DIFF_RELEASE | TPKGINF_OLDER |
				TPKGINF_EQUAL | TPKGINF_UPGRADE | TPKGINF_BLACK);
		
		if(isOnBlackList(pkg->name))
			pkg->flags |= TPKGINF_BLACK;
		
		if(h1){
			cmpr=EVRcmp(pkg->h,h1,1);
			cmp=EVRcmp(pkg->h,h1,0);
			if(cmp!=cmpr)
				pkg->flags|=TPKGINF_DIFF_RELEASE;
			if(cmpr<0){
				pkg->flags|=TPKGINF_OLDER;
			}
			if(cmpr==0){
				pkg->flags|=TPKGINF_EQUAL;
			}
			if(cmpr>0){
				pkg->flags|=TPKGINF_UPGRADE;
			}
			EVRget(h1,1,tmpbuf);
			if(pkg->installedEVR)
				free(pkg->installedEVR);
			
			pkg->installedEVR=strdup(tmpbuf);
			rpmdbFreeIterator(mi);
		}else{
			pkg->installedEVR=NULL;
		}
	}
}

int pkgDescCmp(const void *aa,const void *bb)
{
	const TpkgDescribe *a=aa,*b=bb;
	return strcmp(a->name,b->name);
}

void pkgDescFree(void *aa)
{
	TpkgDescribe *a=aa;
	free(a->name);
	free(a->evr);
	free(a->arch);
	free(a);
}
		

tn_list* allRequired(TPkgSet *pkgs, char *name,TpkgDescribe *pk)
{
	rpmdbMatchIterator req,prov;
	Header h,hp;
	char **requires;
	char *pname;
	int count;
	int i,j;
	tn_list *ret=NULL;
	TpkgDescribe *desc,*next;
	char buf[100];
	
	ret=n_list_new(TN_LIST_UNIQ,pkgDescFree,pkgDescCmp);
	if(!ret)
		return NULL;

	if(pk){
		req=rpmdbInitIterator(pkgs->db,RPMTAG_NAME,name,0);
		if(req){
			h=rpmdbNextIterator(req);
		}else
		{
			printf(_("\nNie zainstalowano podanego pakietu !\n"));
			return NULL;
		}
		headerNVR(h,(const char **)&pname,NULL,NULL);
		pk->name=strdup(pname);
		EVRget(h,1,buf);
		pk->evr=strdup(buf);
		headerGetEntry(h,RPMTAG_ARCH,NULL,(void**)&pname,NULL);
		pk->arch=strdup(pname);
		rpmdbFreeIterator(req);
	}
			

		
	
	j=0;
	next=NULL;
	do{
		if(next)
			name=next->name;
//		printf("Sprawdzam pakiet %s\n",name);

		req=rpmdbInitIterator(pkgs->db,RPMTAG_NAME,name,0);
		if(req){
			h=rpmdbNextIterator(req);
		}else
		{
			printf(_("\nNie zainstalowano podanego pakietu !\n"));
			return NULL;
		}

		rpmHeaderGetEntry(h,RPMTAG_REQUIRENAME,NULL,(void **)&requires,
				&count);

		for(i=0;i<count;i++){
			
			if(!strncmp("rpmlib(",requires[i],strlen("rpmlib(")))
				continue;
			prov=rpmdbInitIterator(pkgs->db,RPMTAG_PROVIDENAME,
					requires[i],0);
			if(!prov)
				prov=rpmdbInitIterator(pkgs->db,
						RPMTAG_BASENAMES,requires[i],0);
			if(!prov){
				continue; /* unsatisfied dep */
			}else{
				hp=rpmdbNextIterator(prov);
			}
			desc=(TpkgDescribe *)malloc(sizeof(TpkgDescribe));
			headerNVR(hp,(const char **)&pname,NULL,NULL);
			desc->name=strdup(pname);
			EVRget(hp,1,buf);
			desc->evr=strdup(buf);
			headerGetEntry(hp,RPMTAG_ARCH,NULL,(void**)&pname,NULL);
			desc->arch=strdup(pname);
			
			n_list_push(ret,desc);
			rpmdbFreeIterator(prov);
//			printf("\n%s -- %s",requires[i],pname);
		}
		rpmdbFreeIterator(req);
	}while((next=(TpkgDescribe *)n_list_nth(ret,j++)));
	

	return ret;
}

int rpmpkgs_isInstalled(TPkgSet *pkgs, char *name, char *evr)
{
	rpmdbMatchIterator req;
	char buf[100];
	Header h;
	req = rpmdbInitIterator(pkgs->db, RPMTAG_NAME, name, 0);
	if(!req)
		return 0;
	h = rpmdbNextIterator(req);
	EVRget(h,1,buf);
	if(strcmp(buf, evr)==0)
		return 1;
	return 0;
}

TpkgDescribe rpmpkgs_getPkgDesc(TPkgSet *pkgs, char *name)
{
	rpmdbMatchIterator req;
	Header h;
	TpkgDescribe ret;
	char buf[100];
	char *pname;
	rpmSetVerbosity(0); 	
	req=rpmdbInitIterator(pkgs->db,RPMTAG_NAME,name,0);
	if(req){
		h=rpmdbNextIterator(req);
	}else
	{
		ret.name=NULL;
		printf("rpmlib: nie ma pakietu w bazie\n");
		return ret;
	}
	
	headerNVR(h,(const char **)&pname,NULL,NULL);
	ret.name=strdup(pname);
	EVRget(h,1,buf);
	printf("EVR == %s\n",buf);
	ret.evr=strdup(buf);
	headerGetEntry(h,RPMTAG_ARCH,NULL,(void**)&pname,NULL);
	ret.arch=strdup(pname);
	rpmdbFreeIterator(req);
	return ret;

}

/* vim:set sw=4 ts=4: */
