#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "net.h"
#include "server_db.h"
#include "sql_quote.h"
#include <stdlib.h>

#define F_ANON		0x00000001
#define F_BEG 		0x00000002
#define F_NOMACH	0x00000004
extern char *mbtEmail;
struct packet_type {
	const char *name;
	void (*handler)(struct user *u, struct phash *pkt, struct phash *ret);
	int flags;
};

#define assert_proto(cond) 				\
	if (!(cond)) {					\
		u->why_killed = "PROTO ERROR";		\
		return;					\
	}

#define get(name) \
	if ((e = ph_find(pkt, name)) == 0) { \
		u->why_killed = "NO " name ; \
		return;	\
	}

/* get dynamic :) */
#define getd(name) \
	if ((e = ph_find(pkt, name)) == 0) { \
		u->why_killed = "required variable missing" ; \
		printf("BRAK #%s#\n",name);	\
		return;	\
	}

	
static int read_pkg(int prefix, struct phash *pkt, TpkgDescribe *r)
{
	char buf[100];
	struct pkt_entry *e;

	snprintf(buf, sizeof(buf), "name_%d", prefix);
	if ((e = ph_find(pkt, buf)) == 0)
		return -1;
	r->name = e->string;
	snprintf(buf, sizeof(buf), "evr_%d", prefix);
	if ((e = ph_find(pkt, buf)) == 0)
		return -1;
	r->evr = e->string;
	snprintf(buf, sizeof(buf), "arch_%d", prefix);
	if ((e = ph_find(pkt, buf)) == 0)
		return -1;
	r->arch = e->string;
	
	return 0;
}

static int read_grade(int prefix, struct phash *pkt, Tgrade *r)
{
	char buf[100];
	struct pkt_entry *e;

	snprintf(buf, sizeof(buf), "feature_no_%d", prefix);
	if ((e = ph_find(pkt, buf)) == 0)
		return -1;
	r->featureNo = e->number;
	snprintf(buf, sizeof(buf), "grade_%d", prefix);
	if ((e = ph_find(pkt, buf)) == 0)
		return -1;
	r->grade = e->number;

	return 0;
}

static void do_get_report(struct user *u, struct phash *pkt, struct phash *ret)
{
	struct pkt_entry *e;
	char *name, *evr, *arch;
	Tgrade *gr;
	int amount,i;
	get("name"); name = e->string;
	get("evr");  evr  = e->string;
	get("arch"); arch = e->string;

	gr = getReport(name, evr, arch, &amount, u->machine_no);
	for(i=0;i< amount; i++){
		char buf[100];
		sprintf(buf,"feature_no_%d", i);
		pkt_add_int(ret, buf, gr[i].featureNo);
		sprintf(buf,"grade_%d", i);
		pkt_add_int(ret, buf, gr[i].grade);
	}
	if(gr)
		free(gr);
	pkt_add_int(ret, "grades_no", amount);
	
}

static void do_add_report(struct user *u, struct phash *pkt, struct phash *ret)
{
	TpkgDescribe pkg, *env;
	Tgrade *grades;
	int env_no, grades_no;
	int i, r;
	struct pkt_entry *e;

	get("env_no"); env_no = e->number;
	get("grades_no"); grades_no = e->number;
	get("name"); pkg.name = e->string;
	get("evr"); pkg.evr = e->string;
	get("arch"); pkg.arch = e->string;
	
	env = ymalloc(env_no * sizeof(TpkgDescribe));
	for (i = 0; i < env_no; i++)
		if (read_pkg(i, pkt, env + i)) {
			u->why_killed = "BAD env";
			yfree(env);
			return;
		}

	grades = ymalloc(grades_no * sizeof(Tgrade));
	for (i = 0; i < grades_no; i++) 
		if (read_grade(i, pkt, grades + i)) {
			u->why_killed = "BAD grade :)";
			yfree(grades);
			yfree(env);
			return;
		}

	r = addReport(u->machine_no, &pkg, env_no, env, grades_no, grades);

	pkt_add_int(ret, "id", r);
}

static void do_add_feature_to_package(struct user *u, struct phash *pkt, struct phash *ret)
{
	int feature_no, package_no;
	struct pkt_entry *e;

	get("feature_no"); feature_no = e->number;
	get("package_name"); package_no = getPackageID(e->string);

	addFeatureToPackage(u->user_no, feature_no, package_no);
}

static void do_get_arch_id(struct user *u, struct phash *pkt, struct phash *ret)
{
	int r;
	struct pkt_entry *e;

	get("name");
	r = getArchID(e->string);
	
	pkt_add_int(ret, "id", r);
}

static void do_get_package_id(struct user *u, struct phash *pkt, struct phash *ret)
{
	int r;
	struct pkt_entry *e;

	get("name");
	r = getPackageID(e->string);
	
	pkt_add_int(ret, "id", r);
}

static void do_add_new_feature(struct user *u, struct phash *pkt, struct phash *ret)
{
	int r;
	struct pkt_entry *e;

	get("description");
	r = addNewFeature(u->user_no, e->string);
	pkt_add_int(ret, "id", r);
}

static void do_register_user(struct user *u, struct phash *pkt, struct phash *ret)
{
	char *nick, *name, *email, *pass;
	int experience_level;
	struct pkt_entry *e;
	int no;

	get("nick"); nick = e->string;
	get("name"); name = e->string;
	get("email"); email = e->string;
	get("experience_level"); experience_level = e->number;
	get("password"); pass = e->string;

	no = registerUser(nick, name, email, experience_level, pass);
	u->user_no = no;
	if (no == 0) 
		u->why_killed = "ALREADY EXIST";
	
	pkt_add_int(ret, "user_no", no);
	
	pkt_add_string(ret, "greet", "Welcome new user to PLD opinion system!");
}

static void do_register_machine(struct user *u, struct phash *pkt, struct phash *ret)
{
	char *name;
	struct pkt_entry *e;

	get("name"); name = e->string;
	u->machine_no=registerMachine(name,u->user_no);
	printf("id maszyny %d\n",u->machine_no);
	
}

static void do_set_machine(struct user *u, struct phash *pkt, struct phash *ret)
{
	struct pkt_entry *e;

	get("name"); u->machine_no = getMachineId(e->string,u->user_no);
	if(u->machine_no == 0)
		u->machine_no=registerMachine(e->string,u->user_no);
	pkt_add_int(ret, "machine_id", u->machine_no);
	printf("id maszyny %d\n",u->machine_no);
}


static void do_get_package_features(struct user *u, struct phash *pkt, struct phash *ret)
{
	int *r, n, *p;
	char *name;
	struct pkt_entry *e;
	char buf[100];

	get("name");
	r = getPackagesFeatures(e->string);
	if (r == 0) {
		pkt_add_string(ret, "error", "No such package");
		return;
	}
	n = 0;
	for (p = r; *p; p++) {
		snprintf(buf, sizeof(buf), "feature_%d", n);
		pkt_add_int(ret, buf, *p);
		/* optimize somewhat, user would ask the names
		 * anyway, and now we have everything handled
		 * in just one packet */
		name = getFeatureName(*p);
		if (name == 0) 
			name = "";
		snprintf(buf, sizeof(buf), "feature_name_%d", n);
		pkt_add_string(ret, buf, name);
		n++;
	}
	pkt_add_int(ret, "no", n);
}

static void do_get_feature(struct user *u, struct phash *pkt, struct phash *ret)
{
	int r;
	struct pkt_entry *e;

	get("name");
	/* PK: approved = 1? */
	r = getFeature(e->string, 0);
	
	pkt_add_int(ret, "id", r);
}


static void do_get_feature_name(struct user *u, struct phash *pkt, struct phash *ret)
{
	char *r;
	struct pkt_entry *e;

	get("id");
	r = getFeatureName(e->number);
	
	pkt_add_string(ret, "name", r ? r : "");
}


static void do_get_max_feature_id(struct user *u, struct phash *pkt, struct phash *ret)
{
	pkt_add_int(ret,"max_feature_id", getMaxFeatureID(0));
} 

static void do_get_packages_for_statistic(struct user *u, struct phash *pkt,
		struct phash *ret)
{
	struct pkt_entry *e;
	int count,i;
	char **pkgs;
	char buf[100];

	
	get("name");
	pkgs=getPackagesForStatistic(e->string,&count);

	pkt_add_int(ret, "no", count);
	for(i=0;i<count;i++){
		snprintf(buf,99,"package_%d",i);
		pkt_add_string(ret, buf, pkgs[i]);
		free(pkgs[i]);
	}
	free(pkgs);
}


char * httpQ(char *str)
{
	static char *ret=NULL;
	char buf[4];
	int i,len;
	len = strlen(str);
	if(ret)
		free(ret);
	ret = malloc(len*3+ 1);
	ret[0]='\0';
	for(i=0;i<len;i++){
		sprintf(buf,"%%%02X",(unsigned char)(str[i]));
		strcat(ret,buf);
	}
	return ret;
		
}	

#define BUG_LINE(name) {get(name); printf("%s=%s\n",name,e->string);}

#define MBT_MAILTO "pawel"

static void do_add_bug_report(struct user *u, struct phash *pkt,
		struct phash *ret)
{
	struct pkt_entry *e;
	int i;
	FILE *mail;
	char *txt;
	char buf[100];
	sprintf(buf, "mail -s MBTAUTO %s", mbtEmail);
	mail = popen(buf,"w");
	fprintf(mail,"c_email=%s\n&\n",httpQ(getEmail(u->user_no)));
	fprintf(mail,"c_pass=%s\n&\n",httpQ(getPassword(u->user_no)));
	get("pkg");
	sprintf(buf,"%d",getPackageID(e->string));
	fprintf(mail,"val_selected_package_id=%s\n&\n",
			httpQ(buf));
	get("title");
	fprintf(mail,"ticket_title=%s\n&\n",httpQ(e->string));

	get("text");
	i = strlen(e->string);
	get("automatic");
	i+=strlen(e->string);
	txt = malloc(i+1);
	if(txt==NULL){
		printf("\nnie udalo sie zaallokowac %d bajtow \n",i+1);
		exit(1);
	}else{printf("zaalokowal %d bajtow\n",i+1);}
	get("text");
	strcpy(txt,e->string);
	get("automatic");
	strcat(txt,e->string);
	fprintf(mail,"ticket_entry=%s\n&\n",httpQ(txt));
	free(txt);

	get("cid");
	fprintf(mail,"ticket_cid=%s\n&\n",httpQ(e->string));
	get("evr");
	fprintf(mail,"evr_input=%s\n&evr_sel=-1\n",httpQ(e->string));
	fprintf(mail,"---\n");
	pclose(mail);

/*	
				
	BUG_LINE("evr");
	BUG_LINE("title");
	BUG_LINE("text");
	BUG_LINE("automatic");
*/
	pkt_add_string(ret,"msg","thank you");
}



static void do_send_list(struct user *u, struct phash *pkt,struct phash *ret,
		char **name, int *id, int count)
{
	int i;
	char buf[1000];

	pkt_add_int(ret,"no",count);
	for(i=0;i<count;i++){
		sprintf(buf,"id_%d",i);
		pkt_add_int(ret, buf,id[i]);
		sprintf(buf,"name_%d",i);
		pkt_add_string(ret, buf, name[i]);
	}
	free(name);
	free(id);
}


static void do_get_machines(struct user *u, struct phash *pkt,struct phash *ret)
{
	int count;
	char **name;
	int *id;

	getMachines(&name, &id, &count, u->user_no);
	do_send_list(u,pkt,ret,name,id,count);
}

static void do_get_dists(struct user *u, struct phash *pkt,struct phash *ret)
{
	int count;
	char **name;
	int *id;

	getDists(&name,&id,&count);
	do_send_list(u,pkt,ret,name,id,count);
}

static void do_get_cathegory(struct user *u, struct phash *pkt,struct phash *ret)
{
	int count;
	char **name;
	int *id;

	getCathegory(&name,&id,&count);
	do_send_list(u,pkt,ret,name,id,count);
}

static void do_get_packages_from_dist(struct user *u, struct phash *pkt,
		struct phash *ret)
{
	struct pkt_entry *e;
	int count,i;
	char **pkgs, **evr;
	char buf[100];
	
	get("dist_id");
	pkgs=getPackagesFromDist(&count, &evr,e->number);

	pkt_add_int(ret, "no", count);
	for(i=0;i<count;i++){
		snprintf(buf,99,"package_%d",i);
		pkt_add_string(ret, buf, pkgs[i]);
		snprintf(buf,99,"evr_%d",i);
		pkt_add_string(ret, buf, evr[i]);
		free(pkgs[i]);
		free(evr[i]);
	}
	free(pkgs);
}

static void do_get_statistic(struct user *u, struct phash *pkt, struct phash *ret)
{

	struct pkt_entry *e;
	int enCount;
	TpkgDescribe pkg,env;
	int *st;
	char buf[100];
	int i,j;
	int *r,*p;

	/* PK: values in e->strings remains valid
	 * until entire packet (pkt) is destroyed,
	 * so there is probably no need to strdup()
	 * it
	 */
	get("package_name");
	pkg.name=strdup(e->string);
	get("package_evr");
	pkg.evr=strdup(e->string);
	get("package_arch");
	pkg.arch=strdup(e->string);

	get("env_no");
	enCount=e->number;

	for(i=0;i<enCount;i++){
		snprintf(buf,99,"enviroment_name_%d",i);
		getd(buf);
		env.name=strdup(e->string);
		snprintf(buf,99,"enviroment_evr_%d",i);
		getd(buf);
		env.evr=strdup(e->string);
		snprintf(buf,99,"enviroment_arch_%d",i);
		getd(buf);
		env.arch=strdup(e->string);

		printf("\n***Pakiet srodowiska %s ver %s, arch %s \n",pkg.name,
				pkg.evr,pkg.arch);
		
		/*  dla kazdego feature  */
		r=getPackagesFeatures(pkg.name);
		for(p=r;*p;p++){
			st=getStatistic(&pkg, &env,*p);
			for(j=0;j<12;j++){
				/*mark_<faeture>_<enviroment>_<sn>*/
				snprintf(buf,99,"mark_%d_%d_%d",*p,i,j);
				pkt_add_int(ret,buf,st[j]);
			}
		}
		free(env.name);free(env.evr);free(env.arch);
		free(r);
	}
		
		
}

static void do_version(struct user *u, struct phash *pkt, struct phash *ret)
{
	char buf[100];
	static int wrap = 1;
	struct pkt_entry *e;

	e = ph_find(pkt, "version");
	if (e == 0 || e->number != 1) {
		u->why_killed = "BAD VERSION";
		return;
	}
	
	pkt_add_int(ret, "version", 1);
	snprintf(buf, 99, "%ld.%ld.%d.%d", 
		(long)getpid(), (long)time(0), u->fd, wrap++);
	u->cookie = ystrdup(buf);
	pkt_add_string(ret, "cookie", buf);
}

static void do_login(struct user *u, struct phash *pkt, struct phash *ret)
{
	struct pkt_entry *e;
	char *pass, *buf;
	char md5[33];

	pkt_add_string(ret, "greet", "Welcome to PLD opinion system!");
	get("email");
	u->user_no = getUid(e->string);
	pkt_add_int(ret, "user_no", getUid(e->string));
	
	pass = getPassword(u->user_no);
	if (pass == 0) {
		u->why_killed = "You don't exist dude, go away!";
		return;
	}
	//u->access = ystrdup(idb_get_string(idb_inode(user, "access", 0)));
	get("md5");
	buf = ymalloc(strlen(pass) + strlen(u->cookie) + 1);
	strcpy(buf, u->cookie);
	strcat(buf, pass);
	pkt_md5(buf, md5);
	if (strcmp(e->string, md5)) 
		u->why_killed = "BAD PASSWD";
	yfree(buf);
}

static struct packet_type types[] = {
	{ "version", do_version, F_ANON|F_BEG | F_NOMACH },
	{ "login", do_login, F_NOMACH | F_ANON },
	{ "register_machine", do_register_machine, F_NOMACH },
	{ "set_machine", do_set_machine, F_NOMACH },
	{ "get_machines",do_get_machines, F_NOMACH},
	{ "get_arch_id", do_get_arch_id, F_ANON | F_NOMACH },
	{ "get_package_id", do_get_package_id, F_ANON | F_NOMACH },
	{ "register_user", do_register_user, F_ANON | F_NOMACH },
	{ "get_package_features", do_get_package_features, F_ANON | F_NOMACH },
	{ "get_feature", do_get_feature, F_ANON | F_NOMACH },
	{ "get_feature_name", do_get_feature_name, F_ANON | F_NOMACH},
	{ "get_max_feature_id",do_get_max_feature_id,F_ANON | F_NOMACH},
	{ "add_feature_to_package", do_add_feature_to_package, F_NOMACH },
	{ "add_new_feature", do_add_new_feature, F_NOMACH },
	{ "add_report", do_add_report, 0 },
	{ "get_report", do_get_report, 0 },
	/* statistic stuff*/
	{ "get_packages_for_statistic", do_get_packages_for_statistic,F_ANON| F_NOMACH},
	{ "get_dists",do_get_dists, F_ANON | F_NOMACH},
	{ "get_cathegory",do_get_cathegory, F_ANON |F_NOMACH},
	{ "get_packages_from_dist",do_get_packages_from_dist,F_ANON | F_NOMACH},
	{ "get_statistic",do_get_statistic,F_ANON | F_NOMACH},
	{ "add_bug_report",do_add_bug_report,0},
	{ 0, 0, 0 }
};

void handle_user(struct user *u)
{
	struct phash *pkt, *ret;
	struct pkt_entry *e;
	const char *err;
	int i;
	
	sql_quote_free();
	
	MYLOG(("handling '%d' [%d]: ", u->user_no, u->fd));
	pkt = pkt_recv(u->fd);
	if (pkt == 0) {
		MYLOG(("EOF\n"));
		kill_user(u);
		return;
	}

	e = ph_find(pkt, "type");

	if (e == 0) {
		err = "NO TYPE";
		goto oops;
	}

	MYLOG(("type=%s ", e->string));

	for (i = 0; types[i].name; i++)
		if (strcmp(types[i].name, e->string) == 0)
			break;
			
	if (types[i].name == NULL) {
		err = "WRONG TYPE";
		goto oops;
	}

	if ((types[i].flags & F_BEG) == 0 && u->cookie == 0) {
		err = "NEED VER";
		goto oops;
	}

	/* we don't support any authentication schame right now,
	 * user just says who he is and we belive her 
	 */
	if ((types[i].flags & F_ANON) == 0 && u->user_no == 0) {
		err = "NEED AUTH";
		goto oops;
	}
	
	if ((types[i].flags & F_NOMACH) == 0 && u->machine_no == 0) {
		err = "NEED MACHINE";
		goto oops;
	}

	

	ret = ph_new();
	types[i].handler(u, pkt, ret);
	pkt_add_string(pkt, "type", e->string);
	err = u->why_killed;
	if (err && strcmp("ALREADY EXIST", err)!=0) {
		pkt_free(ret);
		goto oops;
	}

	pkt_send(u->fd, ret);
	pkt_free(pkt);
	MYLOG(("ok\n"));
	return;
	
oops:
	MYLOG(("%s\n", err));
	pkt_free(pkt);
	kill_user(u);
}
